/* ---------------------------------------------------------------------------------
 * Transmission de signaux entre enfants
 * Jean-Paul Rigault
 * ---------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <assert.h>

/* Pids des fils */
int eldest_pid;		/* ain */
int youngest_pid;	/* cadet */

/* Pige pour SIGUSR1 */
void on_sigusr1(int sig)
{
    assert(sig == SIGUSR1);

    printf("Fils cadet reoit SIGUSR1, envoie SIGUSR2  son an et se termine\n");

    /* Le cadet connait le pid de son ain, car il hrite de l'espace
     * d'adressage de son pre o ce pid a t initialis (la rciproque n'est
     * pas vrai, l'an ne peut connaitre son cadet).
     */
    kill(eldest_pid, SIGUSR2);  
    exit(0); 
}

/* Pige pour SIGUSR2 */
void on_sigusr2(int sig)
{
    assert(sig == SIGUSR2);

    printf("Fils an reoit SIGUSR2 et se termine\n");
    exit(0);
}

int main(){

    struct sigaction sigact1;
    struct sigaction sigact2;
    sigset_t sigusr1_mask;
    sigset_t sigusr2_mask;

    /* On vide les masques: il n'y a rien de spcial  masquer dans le handler */
    sigemptyset(&sigusr1_mask);
    sigemptyset(&sigusr2_mask);

    switch (eldest_pid = fork())
    {
    case -1:
	perror("fork");
	exit(1);
      
    case 0:   
        /* Fils ain */
        printf("Fils an : %d\n", getpid());

        /* Mise en place du pige de SIGUSR2 */
	sigact2.sa_handler = on_sigusr2;
	sigact2.sa_mask = sigusr2_mask;
	sigaction(SIGUSR2, &sigact2, NULL);
	printf("Fils ain en attente\n");    
	sleep(5);
	exit(0);
      
    default:
        /* Le processus pre continue, et cre le cadet */

	switch (youngest_pid = fork())
        {
	case -1:
	    perror("fork");
	    exit(1);
	  
	case 0:	 
            /* Fils cadet */
            printf("Fils cadet : %d\n", getpid());

            /* Mise en place du pige de SIGUSR1 */
	    sigact1.sa_handler = on_sigusr1;
	    sigact1.sa_mask = sigusr1_mask;
	    sigaction(SIGUSR1, &sigact1, NULL);
	    printf("Fils cadet en attente\n");    
	    sleep(5);
	    exit(0);
            break;
	}

        break;
        
    }

    /* Fin du pre */

    /* Il faut laisser le temps au fils cadet de se prparer  "trapper" le signal */
    sleep(1); 

    printf("Pre envoie SIGUSR1 au fils cadet\n");    
    kill(youngest_pid, SIGUSR1);

    /* Le pre attend la fin de ses deux fils */
    wait(0);
    wait(0);	
    return 0;
}
