/* ---------------------------------------------------------------------------------
 * Capture de signaux avec sigaction()
 * Jean-Paul Rigault 
 * ---------------------------------------------------------------------------------
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <assert.h>

/* Fonction de capture des signaux (handler) 
 * -----------------------------------------
 */ 
void on_sig(int sig)
{
    static int count = 0;
  
    switch (sig) 
    {
    case SIGSEGV:
        printf("Signal captur : SIGSEGV\n");
        /* Que faire d'autre que terminer ? */
        exit(1);

    case SIGINT:  
        printf("Signal captur : SIGINT\n");
        if (++count == 5) {
            printf("Reu SIGINT 5 fois\n");
            exit(1);
        }
        break;

    default:
        /* Impossible de passer ici */
        assert(0);
        break;
    }
}  

/* Programme principal
 * -------------------
 * Il commence par attendre SIGINT ou un autre caractre, puis provoque un 
 * SIGSEGV en drfrenant le pointeur nul.
 *
 * Si ce programme est lanc en background, il est immunis contre les signaux
 * mis par des caractres spciaux au terminal (mais pas contre ceux
 * provenant de kill explicites, faits par exemple dans une autre fentre
 * xterm).
 */
int main()
{
    struct sigaction sigact;
    sigset_t sigmask;

    /* Mise en place du pige  signaux */
    sigemptyset(&sigmask);
    sigact.sa_handler = on_sig;
    sigact.sa_mask = sigmask;
    sigaction(SIGINT,  &sigact, NULL);
    sigaction(SIGSEGV,  &sigact, NULL);
		    
    /* On met le programme en attente : attention, sleep() ne conviendrait pas
     * ici (essayez, vous verrez et lisez la page du manuel de sleep (man 3
     * sleep).
     */
    printf("Faites\n"
           "\t- 5 SIGINT pour arrter le programme\n"
           "\t\tsoit '^C' au terminal, si lancement en avant-plan\n"
           "\t\tsoit 'kill -INT %d', si lancement en arrire-plan\n" 
           "\t- envoyez des SIGSEGV (kill -SEGV %d)\n", getpid(), getpid());
    while (1) {}
  
    /* Fin du programme (jamais atteinte en fait) */
    return 0;
}
